@@ -306,8 +306,7 @@ vips__deprecated_open_read( const char *filename, gboolean sequential )
 		/* Yuk. Can't g_object_set() filename since it's after
 		 * construct. Just zap the new filename in.
 		 */
-		VIPS_FREE( image->filename );
-		image->filename = g_strdup( filename );
+		VIPS_SETSTR( image->filename, filename ); 
 
 		return( image );
 	}
