@@ -795,7 +795,7 @@ vips__magick_read_header( const char *filename,
 	printf( "vips__magick_read_header: pinging image ...\n" );
 #endif /*DEBUG*/
 
-	read->image = PingImage( read->image_info, &read->exception );
+	read->image = ReadImage( read->image_info, &read->exception );
 	if( !read->image ) {
 		vips_error( "magick2vips", _( "unable to ping file "
 			"\"%s\"\nlibMagick error: %s %s" ),
@@ -872,7 +872,8 @@ vips__magick_read_buffer_header( const void *buf, const size_t len,
 	printf( "vips__magick_read_buffer_header: pinging blob ...\n" );
 #endif /*DEBUG*/
 
-	read->image = PingBlob( read->image_info, buf, len, &read->exception );
+	read->image = BlobToImage( read->image_info, 
+		buf, len, &read->exception );
 	if( !read->image ) {
 		vips_error( "magick2vips", _( "unable to ping blob\n"
 			"libMagick error: %s %s" ),
