@@ -8,6 +8,10 @@
  * 	- invalidate operation on read error
  * 8/7/17
  * 	- fix DPI mixup, thanks Fosk
+ * 9/9/17
+ * 	- limit max tile width to 30k pixels to prevent overflow in render
+ * 17/9/17 lovell
+ * 	- handle scaling of svg files missing width and height attributes
  */
 
 /*
@@ -89,6 +93,10 @@ typedef struct _VipsForeignLoadSvg {
 	 */
 	double scale;
 
+	/* Scale using cairo when SVG has no width and height attributes.
+	 */
+	double cairo_scale;
+
 	RsvgHandle *page;
 
 } VipsForeignLoadSvg;
@@ -124,21 +132,51 @@ vips_foreign_load_svg_get_flags( VipsForeignLoad *load )
 }
 
 static void
-vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, 
-	VipsImage *out )
+vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 {
 	RsvgDimensionData dimensions;
+	int width;
+	int height;
+	double scale;
 	double res;
 
-	rsvg_handle_set_dpi( svg->page, svg->dpi * svg->scale );
-	rsvg_handle_get_dimensions( svg->page, &dimensions ); 
+	/* Calculate dimensions at default dpi/scale.
+	 */
+	rsvg_handle_set_dpi( svg->page, 72.0 );
+	rsvg_handle_get_dimensions( svg->page, &dimensions );
+	width = dimensions.width;
+	height = dimensions.height;
+
+	/* Calculate dimensions at required dpi/scale.
+	 */
+	scale = svg->scale * svg->dpi / 72.0;
+	if( scale != 1.0 ) {
+		rsvg_handle_set_dpi( svg->page, svg->dpi * svg->scale );
+		rsvg_handle_get_dimensions( svg->page, &dimensions );
+
+		if( width == dimensions.width && height == dimensions.height ) {
+			/* SVG without width and height always reports the same 
+			 * dimensions regardless of dpi. Apply dpi/scale using 
+			 * cairo instead.
+			 */
+			svg->cairo_scale = scale;
+			width = width * scale;
+			height = height * scale;
+		} else {
+			/* SVG with width and height reports correctly scaled 
+			 * dimensions.
+			 */
+			width = dimensions.width;
+			height = dimensions.height;
+		}
+	}
 
 	/* We need pixels/mm for vips.
 	 */
 	res = svg->dpi / 25.4;
 
 	vips_image_init_fields( out, 
-		dimensions.width, dimensions.height,
+		width, height,
 		4, VIPS_FORMAT_UCHAR,
 		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, res, res );
 
@@ -182,7 +220,9 @@ vips_foreign_load_svg_generate( VipsRegion *or,
 	cr = cairo_create( surface );
 	cairo_surface_destroy( surface );
 
-	cairo_translate( cr, -r->left, -r->top );
+	cairo_scale( cr, svg->cairo_scale, svg->cairo_scale );
+	cairo_translate( cr, -r->left / svg->cairo_scale,
+		-r->top / svg->cairo_scale );
 
 	/* rsvg is single-threaded, but we don't need to lock since we're
 	 * running inside a non-threaded tilecache.
@@ -216,6 +256,7 @@ vips_foreign_load_svg_load( VipsForeignLoad *load )
 	int tile_width;
 	int tile_height;
 	int n_lines;
+	int max_tiles;
 
 	/* Use this to pick a tile height for our strip cache.
 	 */
@@ -231,12 +272,19 @@ vips_foreign_load_svg_load( VipsForeignLoad *load )
 		NULL, vips_foreign_load_svg_generate, NULL, svg, NULL ) )
 		return( -1 );
 
-	/* Don't use tilecache to keep the number of calls to
-	 * rsvg_handle_render_cairo() low. Don't thread the cache, we rely on
-	 * locking to keep rsvg single-threaded.
+	/* librsvg starts to fail if any axis in a single render call is over 
+	 * 32767. Use a tilecache so we can render very wide images, though we
+	 * set it up like a linecache. 
+	 *
+	 * Don't thread the cache: we rely on this to keep calls to rsvg 
+	 * single-threaded.
 	 */
-	if( vips_linecache( t[0], &t[1],
+	max_tiles = 3 * VIPS_ROUND_UP( t[0]->Xsize, 30000 ) / 30000;
+	if( vips_tilecache( t[0], &t[1],
+		"tile_width", 30000,
 		"tile_height", tile_height,
+		"max_tiles", max_tiles,
+		"access", VIPS_ACCESS_SEQUENTIAL,
 		NULL ) ) 
 		return( -1 );
 	if( vips_image_write( t[1], load->real ) ) 
@@ -285,6 +333,7 @@ vips_foreign_load_svg_init( VipsForeignLoadSvg *svg )
 {
 	svg->dpi = 72.0;
 	svg->scale = 1.0;
+	svg->cairo_scale = 1.0;
 }
 
 typedef struct _VipsForeignLoadSvgFile {
@@ -470,7 +519,6 @@ vips_foreign_load_svg_is_a_buffer( const void *buf, size_t len )
 	for( i = 0; i < 24; i++ )
 		if( !isascii( str[i] ) )
 			return( FALSE );
-
 	for( i = 0; i < 300 && i < len - 5; i++ )
 		if( g_ascii_strncasecmp( str + i, "<svg", 4 ) == 0 )
 			return( TRUE );
@@ -531,7 +579,7 @@ vips_foreign_load_svg_buffer_init( VipsForeignLoadSvgBuffer *buffer )
 /**
  * vips_svgload:
  * @filename: file to load
- * @out: output image
+ * @out: (out): output image
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
@@ -567,9 +615,9 @@ vips_svgload( const char *filename, VipsImage **out, ... )
 
 /**
  * vips_svgload_buffer:
- * @buf: memory area to load
- * @len: size of memory area
- * @out: image to write
+ * @buf: (array length=len) (element-type guint8): memory area to load
+ * @len: (type gsize): size of memory area
+ * @out: (out): image to write
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
