@@ -7,6 +7,8 @@
  * 	- from header.c
  * 16/7/13
  * 	- leakcheck VipsArea
+ * 16/8/17
+ * 	- validate strings as utf-8 on set
  */
 
 /*
@@ -219,7 +221,7 @@ vips_area_unref( VipsArea *area )
 }
 
 /**
- * vips_area_new: 
+ * vips_area_new: (constructor)
  * @free_fn: (scope async): @data will be freed with this function
  * @data: data will be freed with this function
  *
@@ -276,7 +278,7 @@ vips__type_leak( void )
 		fprintf( stderr, "%d VipsArea alive\n", 
 			g_slist_length( vips_area_all ) );
 		for( p = vips_area_all; p; p = p->next ) {
-			VipsArea *area = (VipsArea *) p->data;
+			VipsArea *area = VIPS_AREA( p->data );
 
 			fprintf( stderr, "\t%p count = %d, bytes = %zd\n", 
 				area, area->count, area->length );
@@ -285,7 +287,7 @@ vips__type_leak( void )
 }
 
 /**
- * vips_area_new_array: 
+ * vips_area_new_array:
  * @type: %GType of elements to store
  * @sizeof_type: sizeof() an element in the array
  * @n: number of elements in the array
@@ -326,7 +328,7 @@ vips_area_free_array_object( GObject **array, VipsArea *area )
 }
 
 /**
- * vips_area_new_array_object:
+ * vips_area_new_array_object: (constructor)
  * @n: number of elements in the array
  *
  * An area which holds an array of %GObject s. See vips_area_new_array(). When
@@ -357,12 +359,12 @@ vips_area_new_array_object( int n )
 }
 
 /**
- * vips_area_get_data:
+ * vips_area_get_data: (method)
  * @area: #VipsArea to fetch from
- * @length: (allow-none): optionally return length in bytes here
- * @n: (allow-none): optionally return number of elements here
- * @type: (allow-none): optionally return element type here
- * @sizeof_type: (allow-none): optionally return sizeof() element type here
+ * @length: (optional): optionally return length in bytes here
+ * @n: (optional): optionally return number of elements here
+ * @type: (optional): optionally return element type here
+ * @sizeof_type: (optional): optionally return sizeof() element type here
  *
  * Return the data pointer plus optionally the length in bytes of an area, 
  * the number of elements, the %GType of each element and the sizeof() each
@@ -507,17 +509,22 @@ transform_save_string_ref_string( const GValue *src_value, GValue *dest_value )
  * @str: (transfer none): string to store
  *
  * Create a new refstring. These are reference-counted immutable strings, used
- * to store string data in vips image metadata.
+ * to store string data in vips image metadata. 
+ *
+ * Strings must be valid utf-8; use blob for binary data.
  *
  * See also: vips_area_unref().
  *
- * Returns: (transfer full): the new #VipsRefString.
+ * Returns: (transfer full): the new #VipsRefString, or NULL on error.
  */
 VipsRefString *
 vips_ref_string_new( const char *str )
 {
 	VipsArea *area;
 
+	if( !g_utf8_validate( str, -1, NULL ) ) 
+		str = "<invalid utf-8 string>";
+
 	area = vips_area_new( (VipsCallbackFn) g_free, g_strdup( str ) );
 
 	/* Handy place to cache this.
@@ -886,6 +893,20 @@ transform_double_array_int( const GValue *src_value, GValue *dest_value )
 	array[0] = g_value_get_double( src_value ); 
 }
 
+static void
+transform_array_double_array_int( const GValue *src_value, GValue *dest_value )
+{
+	int n;
+	double *array_double = vips_value_get_array_double( src_value, &n );
+	int *array_int;
+	int i;
+
+	vips_value_set_array_int( dest_value, NULL, n ); 
+	array_int = vips_value_get_array_int( dest_value, NULL ); 
+	for( i = 0; i < n; i++ )
+		array_int[i] = array_double[i];
+}
+
 GType
 vips_array_int_get_type( void )
 {
@@ -903,6 +924,8 @@ vips_array_int_get_type( void )
 			transform_int_array_int );
 		g_value_register_transform_func( G_TYPE_DOUBLE, type,
 			transform_double_array_int );
+		g_value_register_transform_func( VIPS_TYPE_ARRAY_DOUBLE, type,
+			transform_array_double_array_int );
 	}
 
 	return( type );
@@ -1077,6 +1100,28 @@ transform_int_array_double( const GValue *src_value, GValue *dest_value )
 	array[0] = g_value_get_int( src_value ); 
 }
 
+static void
+transform_array_int_array_double( const GValue *src_value, GValue *dest_value )
+{
+	int n;
+	int *array_int = vips_value_get_array_int( src_value, &n );
+	double *array_double;
+	int i;
+
+	vips_value_set_array_double( dest_value, NULL, n ); 
+	array_double = vips_value_get_array_double( dest_value, NULL ); 
+	for( i = 0; i < n; i++ )
+		array_double[i] = array_int[i];
+}
+
+/* You can set enums from ints, but not doubles. Add a double converter too. 
+ */
+static void
+transform_double_enum( const GValue *src_value, GValue *dest_value )
+{
+	g_value_set_enum( dest_value, g_value_get_double( src_value ) ); 
+}
+
 GType
 vips_array_double_get_type( void )
 {
@@ -1094,13 +1139,17 @@ vips_array_double_get_type( void )
 			transform_double_array_double );
 		g_value_register_transform_func( G_TYPE_INT, type,
 			transform_int_array_double );
+		g_value_register_transform_func( VIPS_TYPE_ARRAY_INT, type,
+			transform_array_int_array_double );
+		g_value_register_transform_func( G_TYPE_DOUBLE, G_TYPE_ENUM,
+			transform_double_enum );
 	}
 
 	return( type );
 }
 
 /**
- * vips_array_image_new:
+ * vips_array_image_new: (constructor)
  * @array: (array length=n): array of #VipsImage
  * @n: number of images
  *
@@ -1138,7 +1187,7 @@ vips_array_image_new( VipsImage **array, int n )
 }
 
 /**
- * vips_array_image_newv:
+ * vips_array_image_newv: (constructor)
  * @n: number of images
  * @...: list of #VipsImage arguments
  *
@@ -1225,7 +1274,7 @@ vips_array_image_new_from_string( const char *string, VipsAccess access )
 }
 
 /**
- * vips_array_image_empty:
+ * vips_array_image_empty: (constructor)
  *
  * Make an empty image array. 
  * Handy with vips_array_image_add() for bindings
@@ -1242,7 +1291,7 @@ vips_array_image_empty( void )
 }
 
 /**
- * vips_array_image_append:
+ * vips_array_image_append: (method)
  * @array: (transfer none): append to this
  * @image: add this
  *
@@ -1282,7 +1331,7 @@ vips_array_image_append( VipsArrayImage *array, VipsImage *image )
 }
 
 /**
- * vips_array_image_get:
+ * vips_array_image_get: (method)
  * @array: the #VipsArrayImage to fetch from
  * @n: length of array
  *
@@ -1325,7 +1374,7 @@ transform_g_string_array_image( const GValue *src_value, GValue *dest_value )
 	g_free( str );
 
 	g_value_set_boxed( dest_value, array_image );
-	vips_area_unref( (VipsArea *) array_image );
+	vips_area_unref( VIPS_AREA( array_image ) );
 }
 
 GType
@@ -1402,12 +1451,17 @@ vips_value_get_save_string( const GValue *value )
  * @str: C string to copy into the GValue
  *
  * Copies the C string into @value.
+ *
+ * @str should be a valid utf-8 string.
  */
 void
 vips_value_set_save_string( GValue *value, const char *str )
 {
 	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_SAVE_STRING );
 
+	if( !g_utf8_validate( str, -1, NULL ) ) 
+		str = "<invalid utf-8 string>";
+
 	g_value_set_boxed( value, str );
 }
 
@@ -1459,6 +1513,8 @@ vips_value_get_ref_string( const GValue *value, size_t *length )
  * vips_ref_string are immutable C strings that are copied between images by
  * copying reference-counted pointers, making them much more efficient than
  * regular %GValue strings.
+ *
+ * @str should be a valid utf-8 string.
  */
 void
 vips_value_set_ref_string( GValue *value, const char *str )
@@ -1502,6 +1558,32 @@ vips_value_set_blob( GValue *value,
 	vips_area_unref( VIPS_AREA( blob ) );
 }
 
+/** 
+ * vips_value_set_blob_free:
+ * @value: (out): GValue to set
+ * @data: pointer to area of memory
+ * @length: length of memory area
+ *
+ * Just like vips_value_set_blob(), but when 
+ * @value is freed, @data will be
+ * freed with g_free(). 
+ *
+ * This can be easier to call for language bindings.
+ *
+ * See also: vips_value_set_blob()
+ */
+void
+vips_value_set_blob_free( GValue *value, void *data, size_t length ) 
+{
+	VipsBlob *blob;
+
+	g_assert( G_VALUE_TYPE( value ) == VIPS_TYPE_BLOB );
+
+	blob = vips_blob_new( (VipsCallbackFn) g_free, data, length );
+	g_value_set_boxed( value, blob );
+	vips_area_unref( VIPS_AREA( blob ) );
+}
+
 /** 
  * vips_value_get_blob:
  * @value: GValue to set
