@@ -136,6 +136,13 @@ main( int argc, char **argv )
 	textdomain( GETTEXT_PACKAGE );
 	setlocale( LC_ALL, "" );
 
+	/* On Windows, argv is ascii-only .. use this to get a utf-8 version of
+	 * the args.
+	 */
+#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+	argv = g_win32_get_command_line();
+#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+
 	context = g_option_context_new( 
 		_( "vipsedit - edit vips file header" ) );
 	main_group = g_option_group_new( NULL, NULL, NULL, NULL, NULL );
@@ -144,7 +151,12 @@ main( int argc, char **argv )
 	g_option_group_set_translation_domain( main_group, GETTEXT_PACKAGE );
 	g_option_context_set_main_group( context, main_group );
 
-	if( !g_option_context_parse( context, &argc, &argv, &error ) ) {
+#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+	if( !g_option_context_parse_strv( context, &argv, &error ) ) 
+#else /*!HAVE_G_WIN32_GET_COMMAND_LINE*/
+	if( !g_option_context_parse( context, &argc, &argv, &error ) ) 
+#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+	{
 		if( error ) {
 			fprintf( stderr, "%s\n", error->message );
 		        g_error_free( error );
@@ -152,7 +164,14 @@ main( int argc, char **argv )
 
 		exit( -1 );
 	}
-	if( argc != 2 ) {
+
+	/* On Windows, argc will not have been updated by
+	 * g_option_context_parse_strv().
+	 */
+	for( argc = 0; argv[argc]; argc++ )
+		;
+
+	if( argc != 2 ) { 
 		fprintf( stderr, _( "usage: %s [OPTION...] vips-file\n" ), 
 			g_get_prgname() );
 		exit( -1 );
@@ -239,6 +258,12 @@ main( int argc, char **argv )
 
 	im_close( im );
 
+	/* We don't free this on error exit, sadly.
+	 */
+#ifdef HAVE_G_WIN32_GET_COMMAND_LINE
+	g_strfreev( argv ); 
+#endif /*HAVE_G_WIN32_GET_COMMAND_LINE*/
+
 	vips_shutdown();
 
 	return( 0 );
